@@ -75,35 +75,35 @@ gxps_page_error_quark (void)
 }
 
 /* Images */
-cairo_surface_t *
+GXPSImage *
 gxps_page_get_image (GXPSPage    *page,
 		     const gchar *image_uri,
 		     GError     **error)
 {
-	cairo_surface_t *surface;
+	GXPSImage *image;
 
 	if (page->priv->image_cache) {
-		surface = g_hash_table_lookup (page->priv->image_cache,
-					       image_uri);
-		if (surface)
-			return cairo_surface_reference (surface);
+		image = g_hash_table_lookup (page->priv->image_cache,
+					     image_uri);
+		if (image)
+			return image;
 	}
 
-	surface = gxps_images_get_image (page->priv->zip, image_uri, error);
-	if (!surface)
+	image = gxps_images_get_image (page->priv->zip, image_uri, error);
+	if (!image)
 		return NULL;
 
 	if (!page->priv->image_cache) {
 		page->priv->image_cache = g_hash_table_new_full (g_str_hash,
 								 g_str_equal,
 								 (GDestroyNotify)g_free,
-								 (GDestroyNotify)cairo_surface_destroy);
+								 (GDestroyNotify)gxps_image_free);
 	}
 
 	g_hash_table_insert (page->priv->image_cache,
 			     g_strdup (image_uri),
-			     cairo_surface_reference (surface));
-	return surface;
+			     image);
+	return image;
 }
 
 /* FixedPage parser */
@@ -206,19 +206,27 @@ gxps_dash_array_parse (const gchar *dash,
 		       guint       *num_dashes_out)
 {
 	gchar  **items;
+        gchar   *stripped_dash;
 	guint    i;
         gdouble *dashes;
         guint    num_dashes;
 
-	items = g_strsplit (dash, " ", -1);
+        stripped_dash = g_strstrip (g_strdup (dash));
+	items = g_strsplit (stripped_dash, " ", -1);
+        g_free (stripped_dash);
 	if (!items)
 		return FALSE;
 
 	num_dashes = g_strv_length (items);
-	dashes = g_malloc (num_dashes * sizeof (gdouble));
+        if (num_dashes % 2 != 0) {
+                g_strfreev (items);
 
+                return FALSE;
+        }
+
+	dashes = g_malloc (num_dashes * sizeof (gdouble));
 	for (i = 0; i < num_dashes; i++) {
-                if (!gxps_value_get_double (items[i], &dashes[i])) {
+                if (!gxps_value_get_double_non_negative (items[i], &dashes[i])) {
                         g_free (dashes);
                         g_strfreev (items);
 
